debug = true
player = {x = 240, y = 300, speed = 150, img = nil}

-- Collision detection taken function from http://love2d.org/wiki/BoundingBox.lua
-- Returns true if two boxes overlap, false if they don't
-- x1,y1 are the left-top coords of the first box, while w1,h1 are its width and height
-- x2,y2,w2 & h2 are the same, but for the second box
function CheckCollision(x1,y1,w1,h1, x2,y2,w2,h2)
	return x1 < x2+w2 and
	x2 < x1+w1 and
	y1 < y2+h2 and
	y2 < y1+h1
end

isAlive = false
lives = 20
livesMax = 20
goodEnding = false
badEnding = false
gameStarted = false

regenInterval = 0
maxRegenInterval = 1000

--timers
canShoot = true
canShootTimerMax = 0.2
createmonsterTimerMax = 0.6
canShootTimer = canShootTimerMax
createmonsterTimer = createmonsterTimerMax

--imageStorage
bulletImg = nil
monsterImg = nil

--entity storage
bullets = {}
monsters = {}
powerups = {}

function love.load(arg)
	windowTitle = 'Shootertale, shoot to get LOVE'

	killSound = love.audio.newSource('assets/music/damage.mp3', 'static')
	bulletSound = love.audio.newSource('assets/music/shoot.wav', 'static')
	regenSound = love.audio.newSource('assets/music/regen.mp3', 'static')

	font = love.graphics.newFont('assets/fonts/8bitoperator.ttf', 14);
	titleFont = love.graphics.newFont('assets/fonts/MonsterFriendFore.otf', 36);
	
	titleMusic = love.audio.newSource('assets/music/title.mp3', 'stream')
	gameOverMusic = love.audio.newSource('assets/music/gameover.mp3', 'stream')
	badGameOverMusic = love.audio.newSource('assets/music/gameover_bad.mp3', 'stream')
	stageMusic = love.audio.newSource('assets/music/bonetrousle.mp3', 'stream')
	goodEndindMusic = love.audio.newSource('assets/music/goodEnding.mp3', 'stream')
	hiMusic = love.audio.newSource('assets/music/thatsgood.mp3', 'stream')
	
	monsterImg = love.graphics.newImage('assets/sprites/monster.png')
	bulletImg = love.graphics.newImage('assets/sprites/redShot.png')
	blackBulletImg = love.graphics.newImage('assets/sprites/blackShot.png')
	pieImg = love.graphics.newImage('assets/sprites/pie.png')
	noodleImg = love.graphics.newImage('assets/sprites/dry_noodle.png')

	playerCentered = love.graphics.newImage('assets/sprites/determination.png')
	player.img = playerCentered
	playerHitImg = love.graphics.newImage('assets/sprites/determination_hit.png')
	playerTrue = love.graphics.newImage('assets/sprites/remember_me.png')

	toEnding = 0

	xp = 0
	lv = 1

	toNextLv = 100
	monsterSpeed = 200
	lifePercentage = 100

	bullet = bulletImg
	powerupImg = pieImg

	playerHit = 0
		
	titleScreenMessages = {
		{lv = 20, message = "Hello"},
		{lv = 25, message = "You really like this huh?"},
		{lv = 30, message = "I really aprecciate your progress"},
		{lv = 35, message = "But don't worry, i can't erase this world"},
		{lv = 40, message = "You can stay here, shooting forever"},
		{lv = 45, message = "Have fun in this infinite nightmare"}
	}
end

function love.update(dt)
	if isAlive then
		toEnding = math.ceil(toEnding + dt/10)
		if toEnding > 15000 and lv == 1 then
			goodEnding = true
		else
			badEnding = true
		end

		if lv >= 20 then
			bullet = blackBulletImg
			powerupImg = noodleImg
		end
		
		regenInterval = regenInterval + math.ceil(1 * dt)
		
		if regenInterval > maxRegenInterval then
			powerupPositionMax = love.graphics.getWidth() - powerupImg:getWidth()
			randomNumber = math.random(10, powerupPositionMax) + 10
			if randomNumber > powerupPositionMax then
				randomNumber = powerupPositionMax
			end
			newPowerup = {x = randomNumber, y = 80, img = powerupImg, spin = 0}
			table.insert(powerups, newPowerup)
			regenInterval = 0
		end

		lifePercentage = (lives * 100) / livesMax
	end

	if not isAlive then
		love.window.setTitle(windowTitle)
	end

	if not isAlive and love.keyboard.isDown('return') then
		love.audio.stop(gameOverMusic)
		love.audio.stop(badGameOverMusic)
		createmonsterTimerMax = 0.6
		bullets = {}
		monsters = {}

		canShootTimer = canShootTimerMax
		createmonsterTimer = createmonsterTimerMax
		monsterSpeed = 200

		player.x = 240 - player.img:getWidth()/2
		player.y = 300
		lives = 20
		livesMax = 20
		regenInterval = 0

		xp = 0
		lv = 1
		isAlive = true
		gameStarted = true
		love.graphics.setBackgroundColor( 0, 0, 0 )

		toEnding = 0
	end

	for i, message in ipairs(titleScreenMessages) do
		if lv >= message.lv then
			love.window.setTitle(message.message)
		end
	end

	if love.keyboard.isDown('escape') then
		love.event.push('quit')
	end

	if isAlive and not goodEnding then

		player.img = playerCentered
		if love.keyboard.isDown('left', 'a') then
			if player.x > 10 then
				player.x = player.x - (player.speed * dt)
			end
		elseif love.keyboard.isDown('right', 'd') then
			if player.x < (love.graphics.getWidth() - player.img:getWidth()) - 14 then
				player.x = player.x + (player.speed * dt)
			end
		end
		if love.keyboard.isDown('up', 'w') then
			if player.y > 80 then
				player.y = player.y - (player.speed * dt)
			end
		elseif love.keyboard.isDown('down', 's') then
			if player.y < (love.graphics.getHeight() - player.img:getHeight()) - 62 then
				player.y = player.y + (player.speed * dt)
			end
		end

		canShootTimer = canShootTimer - (1 * dt)
		if canShootTimer < 0 then
			canShoot = true
		end

		if love.keyboard.isDown('space', 'rctrl', 'lctrl', 'z') and canShoot then
			
			newBullet = {x = player.x + (player.img:getWidth()/2), y = player.y, img = bullet}
			table.insert(bullets, newBullet)
			bulletSound:play()
			canShoot = false
			canShootTimer = canShootTimerMax
		end

		for i, bullet in ipairs(bullets) do
			bullet.y = bullet.y - (500 * dt)

			if (bullet.y) < 80 then
			table.remove(bullets, i)
			end
		end

		createmonsterTimer = createmonsterTimer - (1 * dt)
		if createmonsterTimer < 0 then
			createmonsterTimer = createmonsterTimerMax

			monsterPositionMax = love.graphics.getWidth() - monsterImg:getWidth()
			randomNumber = math.random(10, monsterPositionMax) + 10
			if randomNumber > monsterPositionMax then
				randomNumber = monsterPositionMax
			end
			newmonster = {x = randomNumber, y = 80, img = monsterImg, spin = 0}
			table.insert(monsters, newmonster)
		end

		for i, monster in ipairs(monsters) do
			monster.y = monster.y + (monsterSpeed * dt)
			-- monster.spin = monster.spin + (createmonsterTimerMax/100)
			
			--follow player
			if monster.x < player.x then
				monster.x = monster.x + 50 * dt
			elseif monster.x > player.x then
				monster.x = monster.x - 50 * dt
			end

			--turns to player
			if monster.x < player.x - 10 then
				monster.spin = -15 
			elseif monster.x > player.x + 10 then
				monster.spin = 15
			else
				monster.spin = 0
			end


			if monster.spin >= 360 then
				monster.spin = 0
			end

			if monster.y > 530 then
				table.remove(monsters, i)
			end
		end
	end

	--level up
	if xp >= toNextLv then
		lv = lv + 1
		livesMax = livesMax + math.floor(livesMax / 8)
		lives = lives + math.floor(livesMax / 8)

		if lives >= livesMax then
			lives = livesMax
		end

		monsterSpeed = monsterSpeed + 25

		createmonsterTimerMax = 1 / lv

		toNextLv = math.ceil(toNextLv * 1.25 + monsterSpeed)
	end

	for i, powerup in ipairs(powerups) do

		powerup.y = powerup.y + (monsterSpeed * dt)
		powerup.spin = powerup.spin + 0.001

		if CheckCollision(powerup.x, powerup.y, powerup.img:getWidth(), powerup.img:getHeight(),
			player.x, player.y, player.img:getWidth(), player.img:getHeight())
			and isAlive then
			regenSound:play()
			table.remove(powerups, i)
			lives = lives + livesMax * 0.1
			if lives > livesMax then
				lives = livesMax
			end
		end

		if powerup.y > 530 then
			table.remove(powerups, i)
		end
	end

	--collision detection
	for i, monster in ipairs(monsters) do
		for j, bullet in ipairs(bullets) do
			if CheckCollision(monster.x, monster.y, monster.img:getWidth(), monster.img:getHeight(),
				bullet.x, bullet.y, bullet.img:getWidth(), bullet.img:getHeight())
			and isAlive then
				table.remove(bullets, j)
				table.remove(monsters, i)
				killSound:play()
				xp = xp + 15
			end
		end

		-- monster collision
		if CheckCollision(monster.x, monster.y, monster.img:getWidth(), monster.img:getHeight(),
			player.x, player.y, player.img:getWidth(), player.img:getHeight())
			and isAlive and playerHit == 0 then
			table.remove(monsters, i)
			killSound:play()
			lives = lives -2
			if lives >= 1 then
				playerHit = playerHit + 1 * dt
			else
				isAlive = false
				toEnding = 0
			end
		end

		if playerHit % 2 ~= 0 then
			player.img = playerHitImg
		else
			player.img = playerCentered
		end

		if playerHit > 0 then
			playerHit = playerHit + 1 * dt

			if playerHit > 1 then
				playerHit = 0
				player.img = playerCentered
			end
		end

		if lv >= 20 then
			player.img = playerTrue
		end
	end
end

function love.draw(dt)
	love.graphics.setFont(font)
	if isAlive then
		love.audio.stop(titleMusic)
		love.audio.stop(gameOverMusic)
		love.audio.stop(hiMusic)
		stageMusic:setLooping(true)
		stageMusic:play()
		love.graphics.print("XP " .. xp .. '/' .. toNextLv, 10, 10)
		love.graphics.print("LOVE " .. lv, 10, 30)
		love.graphics.print("HP " .. lives .. '/' .. livesMax, 10, 50)
		love.graphics.setColor(255, 0, 0)
		love.graphics.rectangle('fill', 125, 50, 200, 20)
		love.graphics.setColor(255, 128, 0)
		love.graphics.rectangle('fill', 125, 50, lifePercentage * 2, 20)
		love.graphics.setColor(255, 255, 255)
		love.graphics.rectangle('line', 10, 80, 460, 460)
		love.graphics.setScissor(10, 80, 460, 460)
		love.graphics.draw(player.img, player.x, player.y)
		love.graphics.setScissor()
		if goodEnding then
			love.graphics.clear()
			love.audio.stop(stageMusic)
			goodEndindMusic:setLooping(true)
			goodEndindMusic:play()
			love.graphics.setFont(titleFont)
			love.graphics.printf(
				'FIN',
				love.graphics:getWidth()/2 - 300,
				love.graphics:getHeight()/2 - 50,
				600,
				'center'
			)
			love.graphics.setFont(font)
			love.graphics.printf(
				"You did it\nYou stayed determined and saved your SOUL",
				love.graphics:getWidth()/2 - 150,
				love.graphics:getHeight()/2 + 20, 300,
				'center'
			)
			love.graphics.draw(player.img, love.graphics.getWidth()/2 - player.img:getWidth()/2, 400)
			elseif lv >= 20 then
				love.graphics.setBackgroundColor( 16, 0, 0 )
				love.audio.stop(stageMusic)
				hiMusic:play()
		end
	else
		love.graphics.clear()
		love.audio.stop(stageMusic)
		love.graphics.setFont(titleFont)
		if gameStarted then
			love.graphics.printf('Game Over', love.graphics:getWidth()/2 - 300, love.graphics:getHeight()/2 - 50, 600, 'center')
		end
		if xp > 1 then
			love.graphics.setFont(font)
			love.graphics.printf("Final LOVE " .. lv, love.graphics:getWidth()/2 - 150, love.graphics:getHeight()/2 + 20, 300, 'center')
			badGameOverMusic:setLooping(true)
			badGameOverMusic:play()
		else
			if gameStarted then
				love.graphics.setFont(font)
				love.graphics.printf(
					"You cannot give up just yet... stay determined",
					love.graphics:getWidth()/2 - 150,
					love.graphics:getHeight()/2,
					300,
					'center'
				)
				gameOverMusic:setLooping(true)
				gameOverMusic:play()
			else
				love.graphics.setFont(titleFont)
				love.graphics.printf('ShooteRtale', love.graphics:getWidth()/2 - 200, love.graphics:getHeight()/2 - 50, 400, 'center')
				titleMusic:setLooping(true)
				titleMusic:play()
			end
		end
		
		if not gameStarted then
			love.graphics.setFont(font)
			love.graphics.printf(
				'Press Enter to start',
				love.graphics:getWidth()/2 - 150,
				love.graphics:getHeight()/2 + 50,
				300,
				'center'
			)
			love.graphics.printf(
				'Ctrl/Z/Space to shoot\nWASD/Arrows to move',
				love.graphics:getWidth()/2 - 240,
				love.graphics:getHeight()/2 + 120,
				480,
				'center'
			)
		end

		love.graphics.printf(
			'UnderTale by @tobyfox\nShooterTale by @stsmuniz and @artmuniz',
			love.graphics:getWidth()/2 - 240,
			love.graphics:getHeight()/2 + 250,
			480,
			'center'
		)
	end

	for i, bullet in ipairs(bullets) do
		if isAlive  and not goodEnding then
			-- love.graphics.setScissor(10, 80, 460, 460)
			love.graphics.draw(bullet.img, bullet.x, bullet.y)
			-- love.graphics.setScissor()
		end
	end

	for i, monster in ipairs(monsters) do
		if isAlive and not goodEnding then
			love.graphics.draw(monster.img, monster.x, monster.y, math.deg(monster.spin), 1, 1, 16, 16)
		end
	end

	for i, powerup in ipairs(powerups) do
		if isAlive and not goodEnding then
			love.graphics.draw(powerup.img, powerup.x, powerup.y, math.deg(powerup.spin), 1, 1, 16, 16)
		end
	end
end
--cala a boca